///////////////////////////////////////////////////////////////////////////////////////////////////
// OpenGL Mathematics Copyright (c) 2005 - 2007 G-Truc Creation (www.g-truc.net)
///////////////////////////////////////////////////////////////////////////////////////////////////
// Created : 2006-04-20
// Updated : 2007-04-04
// Licence : This source is under GNU LGPL licence
// File    : glm/gtx/mul.h
///////////////////////////////////////////////////////////////////////////////////////////////////
// Dependency:
// - GLM core
///////////////////////////////////////////////////////////////////////////////////////////////////
// Comment:
// - Extension for Cg compatibility
///////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef __glm_gtx_mul__
#define __glm_gtx_mul__

// Dependency:
#include "../../glm.h"

namespace glm
{
	template <typename genType> genType mulGTX(const genType& a, const genType& b); //!< Product of 2 matrices (From GLM_GTX_mul extension)
	template <typename genType> GLMcolType mulGTX(const genType& m, const GLMcolType& v); //!< Product of a matrix and a vector (From GLM_GTX_mul extension)
	template <typename genType> GLMcolType mulGTX(const GLMcolType& v, const genType& m); //!< Product of a vector and a matrix (From GLM_GTX_mul extension)

    namespace gtx
    {
        namespace mul
        {
            template <typename genType> inline genType mul(const genType& a, const genType& b); //!< Product of 2 matrices (From GLM_GTX_mul extension)
            template <typename genType> inline GLMcolType mul(const genType& m, const GLMcolType& v); //!< Product of a matrix and a vector (From GLM_GTX_mul extension)
            template <typename genType> inline GLMcolType mul(const GLMcolType& v, const genType& m);	//!< Product of a vector and a matrix (From GLM_GTX_mul extension)
        }
    }
}

#define GLM_GTX_mul namespace gtx::mul

/*
    template <typename T> _xmat2<T> mulGTX(const _xmat2<T>& a, const _xmat2<T>& b);
    template <typename T> _xmat3<T> mulGTX(const _xmat3<T>& a, const _xmat3<T>& b);
    template <typename T> _xmat4<T> mulGTX(const _xmat4<T>& a, const _xmat4<T>& b);

    template <typename T> _xvec2<T> mulGTX(const _xmat2<T>& m, const _xvec2<T>& v);
    template <typename T> _xvec3<T> mulGTX(const _xmat3<T>& m, const _xvec3<T>& v);
    template <typename T> _xvec4<T> mulGTX(const _xmat4<T>& m, const _xvec4<T>& v);

    template <typename T> _xvec2<T> mulGTX(const _xvec2<T>& v, const _xmat2<T>& m);
    template <typename T> _xvec3<T> mulGTX(const _xvec3<T>& v, const _xmat3<T>& m);
    template <typename T> _xvec4<T> mulGTX(const _xvec4<T>& v, const _xmat4<T>& m);

    namespace gtx
    {
        namespace mul
        {
            template <typename T> _xmat2<T> mul(const _xmat2<T>& a, const _xmat2<T>& b){return mulGTX(a, b);}
            template <typename T> _xmat3<T> mul(const _xmat3<T>& a, const _xmat3<T>& b){return mulGTX(a, b);}
            template <typename T> _xmat4<T> mul(const _xmat4<T>& a, const _xmat4<T>& b){return mulGTX(a, b);}

            template <typename T> _xvec2<T> mul(const _xmat2<T>& m, const _xvec2<T>& v){return mulGTX(m, v);}
            template <typename T> _xvec3<T> mul(const _xmat3<T>& m, const _xvec3<T>& v){return mulGTX(m, v);}
            template <typename T> _xvec4<T> mul(const _xmat4<T>& m, const _xvec4<T>& v){return mulGTX(m, v);}

            template <typename T> _xvec2<T> mul(const _xvec2<T>& v, const _xmat2<T>& m){return mulGTX(v, m);}
            template <typename T> _xvec3<T> mul(const _xvec3<T>& v, const _xmat3<T>& m){return mulGTX(v, m);}
            template <typename T> _xvec4<T> mul(const _xvec4<T>& v, const _xmat4<T>& m){return mulGTX(v, m);}
        }
    }
*/

#include "mul.inl"

#ifdef GLM_GTX_INCLUDED
namespace glm{using GLM_GTX_mul;}
#endif//GLM_GTX_INCLUDED

#endif//__glm_gtx_mul__
